﻿using System;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Purchasing;

namespace NowGG.Sdk
{

    public interface NowGGPaymentsListener
    {
        void OnNowGGInitSuccess(string response);
        void OnNowGGInitFailed(string error);
        void OnNowGGPurchaseUpdated(string details);
        void OnNowGGPurchaseFailed(int errorCode, string errorMessage);
        void OnNowGGConsumeFinished(string token, int result);

        void OnNowGGAcknowledgeFinished(int result);

        void OnNowGGFetchingAdditionalProductsSuccess(string response);
        void OnNowGGFetchingAdditionalProductsFailed(string error);
    }


    public class NowGGPaymentsSdkManager : AndroidJavaProxy, NowGGPaymentsListener
    {
        AndroidJavaObject nowGGAndroidJavaObj;
        AndroidJavaObject nowGGDetectionJavaObj;

        public Action OnInitSuccess { get; set; }
        public Action<string> OnInitFailed { get; set; }
        public string PaymentId { get; set; }
        public string InGameId { get; set; }

        public delegate PurchaseProcessingResult OnPurchaseCompletedDelegate(PurchasedProduct purchase);
        public OnPurchaseCompletedDelegate OnPurchaseCompleted;
        public Action<int, string> OnPurchaseFailed;

        private Action OnFetchingAdditionalProductsSuccess;
        private Action<string> OnFetchingAdditionalProductsFailed;



        public Product[] products;

        public bool IsInitialized { get; set; }
        public bool IsGuest { get; set; }


        private static NowGGPaymentsSdkManager instance;
        public static NowGGPaymentsSdkManager Instance
        {
            get
            {
                if (instance == null)
                {
                    instance = new NowGGPaymentsSdkManager();
                }
                return instance;
            }
        }

        public static bool IsNowGGIapAvailable()
        {
            bool isNowGGIapAvailable = true;
#if !UNITY_EDITOR
            AndroidJavaClass player = new  AndroidJavaClass("com.unity3d.player.UnityPlayer");
            AndroidJavaObject activity = player.GetStatic<AndroidJavaObject>("currentActivity");

            AndroidJavaClass nowGGClass = new  AndroidJavaClass("gg.now.sdk.NowggPaymentUnitySDK");
            isNowGGIapAvailable = nowGGClass.CallStatic<bool>("checkIAPAvailability",activity);
#endif
            NowGGLogger.Log("is NowGG IAP available.." + isNowGGIapAvailable);
            return isNowGGIapAvailable;
        }

        public NowGGPaymentsSdkManager() : base("gg.now.sdk.common.UnityPaymentsCallBack")
        {
        }

        public void InitializeIap(string appId, string inGameId)
        {
            this.products = new Product[NowGGProductBuilder.Instance.GetProductCount()];
            if (NowGGProductBuilder.Instance.GetProductCount() == 0)
            {
                OnInitFailed("Product id list is empty");
                NowGGLogger.Log("Product id list is empty");
                return;
            }
            nowGGAndroidJavaObj = new AndroidJavaObject("gg.now.sdk.NowggPaymentUnitySDK");

#if !UNITY_EDITOR
            AndroidJavaClass player = new  AndroidJavaClass("com.unity3d.player.UnityPlayer");
            AndroidJavaObject activity = player.GetStatic<AndroidJavaObject>("currentActivity");
            NowGGLogger.Log(appId);
            NowGGLogger.Log(inGameId);
            if(inGameId == null || inGameId.Length == 0 ){
              nowGGAndroidJavaObj.Call("initialize", activity, this, JsonHelper.ToJson(NowGGProductBuilder.Instance.GetProductIdArray()), appId, null);
              IsGuest = false;
            }else{
              nowGGAndroidJavaObj.Call("initialize", activity, this, JsonHelper.ToJson(NowGGProductBuilder.Instance.GetProductIdArray()), appId, inGameId);
              IsGuest = true;
            }
#else
            /// make demo product json string for editor testing
            Product[] products = new Product[NowGGProductBuilder.Instance.GetProductCount()];
            var productIdArray = NowGGProductBuilder.Instance.GetProductIdArray();
            for (int i = 0; i < NowGGProductBuilder.Instance.GetProductCount(); i++)
            {
                products[i] = new Product(productIdArray[i]);
            }
            OnNowGGInitSuccess(JsonHelper.ToJson<Product>(products));
#endif
        }
        public void InitializeIap(string appId)
        {
            this.InitializeIap(appId, null);
        }

        public void OnNowGGInitFailed(string error)
        {
            NowGGLogger.Log("On init fail resp.." + error);
            IsInitialized = false;
            if (OnInitFailed != null)
                OnInitFailed(error);
        }

        public void OnNowGGInitSuccess(string response)
        {
            NowGGLogger.Log("On init success");
            IsInitialized = true;
#if !UNITY_EDITOR
            response = "{\"Items\":" + response + "}";
#endif
            products = JsonHelper.FromJson<Product>(response);
            if (OnInitSuccess != null)
                OnInitSuccess();
        }

        public void PurchaseProduct(string productId, string developerPayload)
        {
            if (IsInitialized == false)
            {
                NowGGLogger.Log("PurchaseProduct FAIL. Not initialized.");
                return;
            }

            bool isValidProduct = false;
            foreach (Product product in products)
            {
                if (string.Equals(productId, product.productId))
                {
                    isValidProduct = true;
                    break;
                }
            }

            if (isValidProduct == false)
            {
                NowGGLogger.Log("Product is not available for purchase");
                return;
            }

            string type = GetProductWithID(productId)?.type;
            NowGGLogger.Log("Purchasing product id " + productId);

#if !UNITY_EDITOR
            nowGGAndroidJavaObj.Call("purchase", productId, type, developerPayload);
#else
            PurchasedProduct[] purchases = { new PurchasedProduct(productId) };
            OnNowGGPurchaseUpdated(JsonHelper.ToJson<PurchasedProduct>(purchases));
#endif
        }

        private void ConsumeProduct(string token)
        {
#if !UNITY_EDITOR
            nowGGAndroidJavaObj.Call("consume", token);
#else
            OnNowGGConsumeFinished("EditorTesting", 0);
#endif
        }

        private void AcknowledgePurchase(string token)
        {
#if !UNITY_EDITOR
            nowGGAndroidJavaObj.Call("acknowledge", token);
#else
            OnNowGGAcknowledgeFinished(0);
#endif
        }

        public void OnNowGGPurchaseUpdated(string details)
        {
            NowGGLogger.Log("OnNowGGProductPurchased: " + details);
#if !UNITY_EDITOR
            details = "{\"Items\":" + details + "}";
#endif
            try
            {
                PurchasedProduct[] purchases = JsonHelper.FromJson<PurchasedProduct>(details);

                foreach (PurchasedProduct purchase in purchases)
                {
                    try
                    {
                        Debug.Log("OnNowGGPurchaseUpdated : " + purchase.productId);
                        Debug.Log("OnNowGGPurchaseUpdated : " + purchase.skuDetails);
                        PurchaseProcessingResult result = OnPurchaseCompleted(purchase);
                        if (result == PurchaseProcessingResult.Complete
                            && (NowGGProductBuilder.Instance.GetProductType(purchase.productId) == ProductType.Consumable))
                        {
                            NowGGLogger.Log($"Consuming product: {purchase.productId}");
                            try
                            {
                                //ServerVerification(purchase.purchaseToken, purchase.productId, "KRW");
                                ConsumeProduct(purchase.purchaseToken);
                            }
                            catch (Exception e)
                            {
                                NowGGLogger.Log("Error consuming purchase: " + e.Message);
                            }
                        }
                        else if (result == PurchaseProcessingResult.Complete && !purchase.isAcknowledged)
                        {
                            try
                            {
                                AcknowledgePurchase(purchase.purchaseToken);
                            }
                            catch (Exception e)
                            {
                                NowGGLogger.Log("Error acknowledging purchase: " + purchase.purchaseToken + " " + e.Message);
                            }

                        }
                    }
                    catch (Exception e)
                    {
                        NowGGLogger.Log("Error processing purchase: " + e.Message);
                    }

                }
            }
            catch (Exception e)
            {
                NowGGLogger.Log("Error parsing purchase details: " + e.Message);
            }

        }

        public void OnNowGGPurchaseFailed(int errorCode, string errorMessage)
        {
            NowGGLogger.Log($"OnNowGGPurchaseFailed called in Unity, errorCode: {errorCode} and msg: {errorMessage}");
            if (OnPurchaseFailed != null)
                OnPurchaseFailed(errorCode, errorMessage);
        }

        public void OnNowGGConsumeFinished(string token, int result)
        {
            NowGGLogger.Log($"onNowGGConsumeFinished called in Unity, token: {token} and result: {result}");
        }

        public void OnNowGGAcknowledgeFinished(int result)
        {
            NowGGLogger.Log($"OnNowGGAcknowledgeFinished called in Unity, result: {result}");
        }

        public void OnNowGGFetchingAdditionalProductsSuccess(string response)
        {
            NowGGLogger.Log("On fetching additional product success");
            IsInitialized = true;
#if !UNITY_EDITOR
            response = "{\"Items\":" + response + "}";
#endif
            products = JsonHelper.FromJson<Product>(response);

            if (OnFetchingAdditionalProductsSuccess != null)
                OnFetchingAdditionalProductsSuccess();
        }

        public void OnNowGGFetchingAdditionalProductsFailed(string error)
        {
            if (OnFetchingAdditionalProductsFailed != null)
                OnFetchingAdditionalProductsFailed(error);
        }

        public void ConfirmPendingPurchase(string purchasetoken, ProductType productType)
        {
            if (productType == ProductType.Consumable)
            {
                ConsumeProduct(purchasetoken);
            }
            else
            {
                AcknowledgePurchase(purchasetoken);
            }
        }

        public void ConfirmPendingPurchase(PurchasedProduct purchasedProduct)
        {
            if (NowGGProductBuilder.Instance.GetProductType(purchasedProduct.productId) == ProductType.Consumable)
            {
                ConsumeProduct(purchasedProduct.purchaseToken);
            }
            else
            {
                AcknowledgePurchase(purchasedProduct.purchaseToken);
            }

        }

        public void QueryPendingPurchases()
        {
            nowGGAndroidJavaObj.Call("queryPendingPurchases");
        }

        public void FetchAdditionalProducts(Dictionary<string, ProductType> additionalProductDict, Action successCallback, Action<string> failCallback)
        {
            OnFetchingAdditionalProductsSuccess = successCallback;
            OnFetchingAdditionalProductsFailed = failCallback;
            NowGGProductBuilder.Instance.AddAdditionalProducts(additionalProductDict);
            nowGGAndroidJavaObj.Call("fetchAdditionalProducts", JsonHelper.ToJson(NowGGProductBuilder.Instance.GetProductIdArray()));
        }

        public Product GetProductWithID(string productId)
        {
            foreach (Product product in products)
            {
                if (string.Equals(productId, product.productId))
                {
                    return product;
                }
            }
            return null;
        }

        public string GetServerVerificationJSON(PurchaseEventArgs args)
        {
            string receiptString = args.purchasedProduct.receipt;
            var receiptDict = (Dictionary<string, object>)MiniJson.JsonDecode(receiptString);
            string payloadString = (string)receiptDict["Payload"];
            var payloadDict = (Dictionary<string, object>)MiniJson.JsonDecode(payloadString);
            //Stuff from json object
            string jsonString = (string)payloadDict["json"];
            var jsonDict = (Dictionary<string, object>)MiniJson.JsonDecode(jsonString);
            string purchaseTokenString = (string)jsonDict["purchaseToken"];
            string productIdString = (string)jsonDict["productId"];
            return "{\"purchaseToken\" : \"" + purchaseTokenString + "\", \"productId\" : \"" + productIdString
            + "\", \"appId\" : \"" + NowGG.Sdk.NowGGPaymentsSdkManager.Instance.PaymentId + "\", \"currencyCode\" : \"" + args.purchasedProduct.metadata.isoCurrencyCode + "\", \"storeType\" : \"" + NowGGPaymentsSdkManager.GetStoreType() + "\"}";
        }

        public static bool IsNowGGPlatform()
        {
            bool isNowGGPlatform = false;
#if !UNITY_EDITOR
            AndroidJavaClass player = new  AndroidJavaClass("com.unity3d.player.UnityPlayer");
            AndroidJavaObject activity = player.GetStatic<AndroidJavaObject>("currentActivity");

            AndroidJavaClass nowGGClass = new  AndroidJavaClass("gg.now.sdk.NowggDetectionSDK");
            isNowGGPlatform = nowGGClass.CallStatic<bool>("isNowGGPlatform", activity);
#endif
            NowGGLogger.Log("is NowGG Platform.." + isNowGGPlatform);
            return isNowGGPlatform;
        }

        public static string GetNowGGDeviceType()
        {
            string deviceType = "NA";
#if !UNITY_EDITOR
            AndroidJavaClass player = new  AndroidJavaClass("com.unity3d.player.UnityPlayer");
            AndroidJavaObject activity = player.GetStatic<AndroidJavaObject>("currentActivity");

            AndroidJavaClass nowGGClass = new  AndroidJavaClass("gg.now.sdk.NowggDetectionSDK");
            deviceType = nowGGClass.CallStatic<string>("getDeviceTypeNowGG", activity);
#endif
            NowGGLogger.Log("NowGG device type.." + deviceType);
            return deviceType;
        }

        public static string GetStoreType()
        {
            Debug.Log("GetStoreType() called");
#if !UNITY_EDITOR
            AndroidJavaClass nowGGClass = new  AndroidJavaClass("gg.now.sdk.NowggPaymentUnitySDK");
            return nowGGClass.CallStatic<string>("getStoreType");
#else
            return "NOT_INIT";
#endif
        }

        public void RecordGameEvent(string eventName, string eventData, string userId)
        {
#if !UNITY_EDITOR
            nowGGAndroidJavaObj.Call("recordGameEvent", eventName, eventData, userId);
#endif
        }
    }
}
