using System.Collections;
using System.Collections.Generic;
using UnityEngine;


namespace gg.now.wallet.Sdk
{
    public delegate void VoidCallback();
    public delegate void StringCallback(string parameter);
    delegate void OnSettledSessionsReceived(List<Session> settledSessions);
    public delegate void OnPairingsReceived(List<Pairing> pairings);
    public delegate void OnNftsReceived(List<Nft> nfts);
    public delegate void OnTransactionsReceived(TransactionInfo transaction);

    /// <summary>
    /// Interface defining contract on callbacks on interaction with wallet.
    /// </summary>
    public interface NowGGWalletDelgate
    {
        /// <summary>
        /// triggered when session request is approved by the wallet.
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="approvedSession">session details including topic, wallet address</param>
        /// <param name="user">user info of wallet holder.</param>
        void OnSessionApproved(ApprovedSession approvedSession, User user);

        /// <summary>
        /// triggered when session request is rejected by the wallet.
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="rejectedSession">session info for which request was rejected.</param>
        void OnSessionRejected(RejectedSession rejectedSession);

        /// <summary>
        /// triggered when session info is updated.
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="updatedSession">updated session information</param>
        void OnSessionUpdate(UpdatedSession updatedSession);

        /// <summary>
        /// triggered when session is deleted.
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="deletedSession">session info which was deleted.</param>
        void OnSessionDelete(DeletedSession deletedSession);

        //Responses

        // Utils
        /// <summary>
        /// trigged when the connection state changes.
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="isAvailable">flag which tells if the connection is available.</param>
        void OnConnectionStateChange(bool isAvailable);
        /// <summary>
        /// trigged when there is any error
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="error">error info</param>
        void OnError(string error);

        /// <summary>
        /// trigged when token purchase is successful
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="result">result received from wallet</param>
        /// <param name="extra">extra data received from wallet</param>
        void OnTokenPurchaseSuccess(JsonRpcResponse result, string extra);


        /// <summary>
        /// trigged when token purchase failed
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="error">error received from wallet</param>
        /// <param name="extra">extra data received from wallet</param>
        void OnTokenPurchaseError(string error, string extra);


        /// <summary>
        /// trigged when error occured while requesting for send transaction operation
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="error">error information</param>
        /// <param name="extra">extra data provided along with error</param>
        void OnTransactionSendError(string error, string extra);

        /// <summary>
        /// trigged when request was successfully made for transaction to wallet
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="txnHash">Hash of transaction received from wallet</param>
        /// <param name="caipChainId">Chain agnostic improvment proposal's Chain id</param>
        /// <param name="extra">extra data provided from wallet</param>
        void OnTransactionSendSuccess(string txnHash, string caipChainId, string extra);
        /// <summary>
        /// trigged when transaction was completed successful 
        /// <returns>
        /// void
        /// </returns>
        /// <param name="transaction">transaction info object</param>
        /// <param name="extra">extra data provided by wallet</param>
        void OnTransactionCompletedSuccess(TransactionInfo transaction, string extra);

        /// <summary>
        /// trigged when transaction completed with error post wallet approval request
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="error">error information</param>
        /// <param name="extra">extra data provided along with error</param>
        void OnTransactionCompletedError(string error, string extra);
    }

}

