#nullable enable

using System;
using System.Collections.Generic;
using System.Net;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Unity.VisualScripting;
using UnityEngine;
using static UnityEngine.EventSystems.EventTrigger;

namespace gg.now.wallet.Sdk
{

    /// <summary>
    /// WalletManager provides public methods for the dapp to interact with the web3-wallets.
    /// </summary>

    public class NowGGWalletSdkManager : AndroidJavaProxy, DappDelegate
    {
        AndroidJavaObject? walletSdkJavaObject;
        NowGGWalletDelgate? walletDelgate;
        private bool isInitialised = false;
        private bool isConnected = false;




        private static NowGGWalletSdkManager? instance;

        /// <summary>
        /// Returns instance of the class
        /// </summary>
        public static NowGGWalletSdkManager Instance
        {
            get
            {
                Debug.Log("WalletSDKManager: getInstance");
                if (instance == null)
                {
                    instance = new NowGGWalletSdkManager();
                }
                return instance;
            }
        }



        public NowGGWalletSdkManager() : base("gg.now.wallet.sdk.WalletSdkDelegate")
        {
        }

        /// <summary>
        /// Initializes the Dapp sdk with provided arguments
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="apiKey">api key provided by now.gg. </param>
        /// <param name="appId">dapp specific id, which will be received upon registering with now.gg</param>
        /// <param name="walletDelegate">delegate implementation for receiving events in dapp.</params>
        /// <param name="onDone">callback function that will be invoked once initialisation of sdk is complete</param>
        /// <param name="onFailed">callback function that will be invoked once initialisation of sdk is failed</param>
        public void Initialize(string apiKey, string appId, NowGGWalletDelgate walletDelgate, VoidCallback onDone, StringCallback onFailed)
        {
            CheckIfRunningInEditor();


            if (!isInitialised)
            {
                AndroidJavaClass player = new AndroidJavaClass("com.unity3d.player.UnityPlayer");
                AndroidJavaObject activity = player.GetStatic<AndroidJavaObject>("currentActivity");
                var application = activity.Call<AndroidJavaObject>("getApplication");
                StringCallback successCallback = (message) =>
                {
                    onDone();
                };

                this.walletDelgate = walletDelgate;
                if (walletDelgate == null)
                {
                    throw new ArgumentNullException("walletDelegate cannot be null");
                }
                walletSdkJavaObject = new AndroidJavaObject("gg.now.wallet.sdk.WalletSdk", activity, apiKey, appId, new Listener(successCallback, onFailed), this);

                isInitialised = true;
            }

        }

        /// <summary>
        /// Requests connection to Now.gg Wallet.
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="caipChainId">Chain agnostic improvement proposal's chain id(https://github.com/ChainAgnostic/CAIPs/blob/master/CAIPs/caip-2.md#syntax)</param>
        /// <param name="onPairingInvoked">called when pairing is invoked</param>
        /// <param name="onError">called in case of an error</param>
        /// <exception cref = "WalletSdkNotInitialisedException"> This exception is thrown if the sdk is not initialised</exception>
        /// <exception cref = "SocketNotConnectedException"> This exception is thrown if connection state is not available</exception>
        public void Connect(String caipChainId, VoidCallback onPairingInvoked, StringCallback onError)
        {
            CheckIfRunningInEditor();
            CheckIfSdkInitiliased();
            CheckIfSocketIsConnected();
            AndroidJavaClass player = new AndroidJavaClass("com.unity3d.player.UnityPlayer");
            AndroidJavaObject activity = player.GetStatic<AndroidJavaObject>("currentActivity");

            walletSdkJavaObject?.Call("connect", caipChainId, activity, new PairingInvokedCallback(onPairingInvoked), new ErrorCallback(onError));
        }

        /// <exception cref = "WalletSdkNotInitialisedException" > This exception is thrown if the sdk is not initialised</exception>
        public void Disconnect(StringCallback onError)
        {
            CheckIfRunningInEditor();
            CheckIfSdkInitiliased();
            walletSdkJavaObject?.Call("disconnect", new ErrorCallback(onError));
        }


        /// <summary>
        /// Request for transaction approval to connected wallet.
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="toAddress">wallet address where transaction value should be sent</param>
        /// <param name="value">value to be sent</param>
        /// <param name="data">data to be sent along with request</param>
        /// <param name="caipChainId">Chain agnostic improvement proposal's chain id(https://github.com/ChainAgnostic/CAIPs/blob/master/CAIPs/caip-2.md#syntax)</param>
        /// <param name="extra">extra info to be passed for transaction</param>
        /// <exception cref = "WalletSdkNotInitialisedException">This exception is thrown if the sdk is not initialised</exception>
        /// <exception cref = "SocketNotConnectedException">This exception is thrown if connection state is not available</exception>
        public void SendTransaction(string toAddress, string value, string data, string caipChainId, string extra)
        {
            CheckIfRunningInEditor();
            CheckIfSdkInitiliased();
            CheckIfSocketIsConnected();
            walletSdkJavaObject?.Call("sendTransaction", toAddress, value, data, caipChainId, extra);
        }

        /// <summary>
        /// Request the wallet to initiate buying token procedure.
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="erc20TokenContractAddress">ERC20 contract address of token which needs to be bought</param>
        /// <param name="value">value of token to be bought.</param>
        /// <param name="caipChainId">Chain agnostic improvement proposal's chain id(https://github.com/ChainAgnostic/CAIPs/blob/master/CAIPs/caip-2.md#syntax)</param>
        /// <param name="extra">extra info to be passed for transaction</param>
        /// <exception cref = "WalletSdkNotInitialisedException">This exception is thrown if the sdk is not initialised</exception>
        /// <exception cref = "SocketNotConnectedException">This exception is thrown if connection state is not available</exception>
        public void BuyToken(string erc20TokenContractAddress, string value, string caipChainId, string extra)
        {
            CheckIfRunningInEditor();
            CheckIfSdkInitiliased();
            CheckIfSocketIsConnected();
            walletSdkJavaObject?.Call("buyToken", value, caipChainId, erc20TokenContractAddress, extra);
        }

        /// <summary>
        /// Requests list of NFTs for current user.
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="caipChainId">chainId id on which request should be made, in CAIP format(chain id for which balance is requested in CAIP format(https://github.com/ChainAgnostic/CAIPs/blob/master/CAIPs/caip-2.md#syntax).</param>
        /// <param name="address">wallet address of the current user</param>

        /// <param name="onNftsReCeived">called when result is available for NFTs</param>
        /// <param name="onError">called in case of an error</param>
        /// <exception cref = "WalletSdkNotInitialisedException"> This exception is thrown if the sdk is not initialised</exception>
        public void GetNfts(string caipChainId, string address, OnNftsReceived onNftsReceived, StringCallback onError)
        {
            CheckIfRunningInEditor();
            CheckIfSdkInitiliased();
            StringCallback onSuccess = (payLoad) =>
            {
                var nfts = JsonConvert.DeserializeObject<List<Nft>>(payLoad);
                onNftsReceived(nfts);
            };


            walletSdkJavaObject?.Call("getNftBalances", caipChainId, address, new Listener(onSuccess, onError));
        }

        /// <summary>
        /// Retrieves token balance for current user
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="erc20TokenContractAddress">ERC20 contract address of token whose balance needs to be retrieved.</param>
        /// <param name="caipChainId">chain id for which balance is requested in CAIP format(https://github.com/ChainAgnostic/CAIPs/blob/master/CAIPs/caip-2.md#syntax).</param>
        /// <param name="address">current user's wallet address.</param>
        /// <param name="onSuccess">callback when result is available.</param>
        /// <param name="onError">called in case of an error</param>
        /// <exception cref = "WalletSdkNotInitialisedException">This exception is thrown if the sdk is not initialised</exception>
        public void GetTokenBalance(string erc20TokenContractAddress, string caipChainId, string address, StringCallback onSuccess, StringCallback onError)
        {
            CheckIfRunningInEditor();
            CheckIfSdkInitiliased();
            walletSdkJavaObject?.Call("getTokenBalance", erc20TokenContractAddress, caipChainId, address, new Listener(onSuccess, onError));
        }

        /// <summary>
        /// Retrieves the transaction information for the hash provided.
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="chainId">chain id where transaction was made in CAIP format(https://github.com/ChainAgnostic/CAIPs/blob/master/CAIPs/caip-2.md#syntax).</param>
        /// <param name="transactionHash">transaction hash for which info is being requested.</param>
        /// <param name="onTransactionsReceived">callback when result is available.</param>
        /// <param name="onError">called in case of an error</param>
        /// <exception cref = "WalletSdkNotInitialisedException">This exception is thrown if the sdk is not initialised</exception>
        public void GetTransactionInfo(string caipChainId, string transactionHash, OnTransactionsReceived onTransactionsReceived, StringCallback onError)
        {
            CheckIfRunningInEditor();
            CheckIfSdkInitiliased();
            StringCallback onSuccess = (payLoad) =>
            {
                var transaction = JsonConvert.DeserializeObject<TransactionInfo>(payLoad);
                onTransactionsReceived(transaction);
            };
            walletSdkJavaObject?.Call("getTransactionDetails", caipChainId, transactionHash, new Listener(onSuccess, onError));
        }

        /// <summary>
        /// Pings the connected wallet.
        /// </summary>
        /// <returns>
        /// void
        /// </returns>
        /// <param name="sessionTopic">session where ping request will be made.</param>
        /// <param name="onSuccess">callback when ping is successful.</param>
        /// <param name="onError">called in case of an error</param>
        /// <exception cref = "WalletSdkNotInitialisedException">This exception is thrown if the sdk is not initialised</exception>
        public void Ping(StringCallback onPingSuccess, StringCallback onPingError)
        {
            CheckIfRunningInEditor();
            CheckIfSdkInitiliased();
            walletSdkJavaObject?.Call("ping", new PingCallback(onPingSuccess, onPingError));
        }


        public void OnSessionApproved(string approvedSession, string userJson)
        {
            Debug.Log("WalletSDKManager: DappDelegate:OnSessionApproved:" + approvedSession);
            var approvedSessionModel = JsonConvert.DeserializeObject<ApprovedSession>(approvedSession);
            var userModel = JsonConvert.DeserializeObject<User>(userJson);
            walletDelgate?.OnSessionApproved(approvedSessionModel, userModel);
        }

        public void OnConnectionStateChange(bool isAvailable)
        {
            Debug.Log("WalletSDKManager: DappDelegate:OnConnectionStateChange:" + isAvailable);
            walletDelgate?.OnConnectionStateChange(isAvailable);
            isConnected = isAvailable;
        }

        public void OnSessionRejected(string rejectedSession)
        {
            Debug.Log("WalletSDKManager: DappDelegate:OnSessionRejected:" + rejectedSession);
            var rejectedSessionModel = JsonConvert.DeserializeObject<RejectedSession>(rejectedSession);
            walletDelgate?.OnSessionRejected(rejectedSessionModel);
        }

        public void OnSessionUpdate(string updatedSession)
        {
            Debug.Log("WalletSDKManager: DappDelegate:OnSessionUpdate:" + updatedSession);
            var updatedSessionModel = JsonConvert.DeserializeObject<UpdatedSession>(updatedSession);
            walletDelgate?.OnSessionUpdate(updatedSessionModel);
        }

        public void OnSessionDelete(string deletedSession)
        {
            Debug.Log("WalletSDKManager: DappDelegate:OnSessionDelete:" + deletedSession);
            var deletedSessionModel = JsonConvert.DeserializeObject<DeletedSession>(deletedSession);
            walletDelgate?.OnSessionDelete(deletedSessionModel);
        }

        public void OnTransactionSendSuccess(string txnHash, string caipChainId, string extra)
        {
            Debug.Log("WalletSDKManager: DappDelegate:OnTransactionSendSuccess:" + txnHash);
            walletDelgate?.OnTransactionSendSuccess(txnHash, caipChainId, extra);
        }

        public void OnTokenPurchaseSuccess(string response, string extra)
        {
            Debug.Log("WalletSDKManager: DappDelegate:OnTokenPurchaseSuccess:" + response);
            var responseModel = JsonConvert.DeserializeObject<SessionRequestResponse>(response);
            if (responseModel != null)
            {
                walletDelgate?.OnTokenPurchaseSuccess(responseModel.result, extra);
            }
            else
            {
                walletDelgate?.OnTokenPurchaseError("Error parsing response", extra);
            }

        }

        public void OnError(string error)
        {
            Debug.Log("WalletSDKManager: DappDelegate:OnError:" + error);
            walletDelgate?.OnError(error);
        }

        public void OnTransactionCompletedSuccess(string transactionResponse, string extra)
        {
            Debug.Log("WalletSDKManager: DappDelegate:OnTransactionCompletedSuccess:" + transactionResponse);
            var transaction = JsonConvert.DeserializeObject<TransactionInfo>(transactionResponse);
            walletDelgate?.OnTransactionCompletedSuccess(transaction, extra);
        }

        public void OnTransactionSendError(string error, string extra)
        {
            Debug.LogError("WalletSDK:DappDelegate:OnTransactionSendError:" + error);
            walletDelgate?.OnTransactionSendError(error, extra);
        }

        public void OnTransactionCompletedError(string error, string extra)
        {
            Debug.LogError("WalletSDK:DappDelegate:OnTransactionCompletedError:" + error);
            walletDelgate?.OnTransactionCompletedError(error, extra);
        }

        public void OnTokenPurchaseError(string error, string extra)
        {
            Debug.LogError("WalletSDK:DappDelegate:OnTokenPurchaseError:" + error);
            walletDelgate?.OnTokenPurchaseError(error, extra);
        }

        private void CheckIfRunningInEditor()
        {
            if (Application.isEditor)
            {
                Debug.Log("Operation not supported in editor, please launch on android");
            }
        }

        private void CheckIfSdkInitiliased()
        {
            if (!isInitialised)
            {
                Debug.LogError("Wallet Sdk not initialised.");
                throw new WalletSdkNotInitialisedException();
            }
        }

        private void CheckIfSocketIsConnected()
        {
            if (!isConnected)
            {
                Debug.LogError("Wallet Sdk not initialised.");
                throw new SocketNotConnectedException();
            }
        }
    }

    public interface DappDelegate
    {
        public void OnSessionApproved(string approvedSession, string userJson);
        public void OnSessionRejected(string rejectedSession);
        public void OnSessionUpdate(string updatedSession);
        public void OnSessionDelete(string sessionEvent);


        //Responses

        public void OnTransactionSendSuccess(string txnHash, string caipChainId, string extra);
        public void OnTransactionSendError(string error, string extra);
        public void OnTransactionCompletedSuccess(string transactionResponse, string extra);
        public void OnTransactionCompletedError(string error, string extra);
        public void OnTokenPurchaseSuccess(string response, string extra);
        public void OnTokenPurchaseError(string error, string extra);

        // Utils
        public void OnConnectionStateChange(bool isAvailable);
        public void OnError(string error);
    }

    class PairingInvokedCallback : AndroidJavaProxy
    {
        VoidCallback onPairingInvoked;

        public PairingInvokedCallback(VoidCallback onPairingInvoked) : base("gg.now.wallet.sdk.listeners.PairingInvokedCallback")
        {
            this.onPairingInvoked = onPairingInvoked;
        }

        public void OnSuccess()
        {
            Debug.Log("Pairing Invoked");
            onPairingInvoked();
        }
    }

    class ErrorCallback : AndroidJavaProxy
    {

        public StringCallback onError;

        public ErrorCallback(StringCallback onError) : base("gg.now.wallet.sdk.listeners.ErrorCallback")
        {
            this.onError = onError;
        }

        public void OnError(string errorMessage)
        {
            Debug.Log(errorMessage);
            onError(errorMessage);
        }
    }


    class PingCallback : AndroidJavaProxy
    {

        public StringCallback onPingSuccess;
        public StringCallback onPingError;

        public PingCallback(StringCallback onPingSuccess, StringCallback onPingError) : base("gg.now.wallet.sdk.listeners.PingCallback")
        {
            this.onPingSuccess = onPingSuccess;
            this.onPingError = onPingError;


        }

        public void OnSuccess(string sessionTopic)
        {
            Debug.Log(sessionTopic);
            onPingSuccess(sessionTopic);
        }

        public void OnError(string errorMessage)
        {
            Debug.Log(errorMessage);
            onPingError(errorMessage);
        }
    }


    class GetSettledSessionCallback : AndroidJavaProxy
    {
        public OnSettledSessionsReceived onSettledSessionsReceived;

        public GetSettledSessionCallback(OnSettledSessionsReceived onSettledSessionsReceived) : base("gg.now.wallet.sdk.listeners.GetSettledSessionCallback")
        {
            this.onSettledSessionsReceived = onSettledSessionsReceived;

        }

        public void OnSettledSessionsReceived(string settledSessions)
        {
            var sessions = JsonConvert.DeserializeObject<List<Session>>(settledSessions);
            onSettledSessionsReceived(sessions);
        }
    }

    class GetPairingsCallback : AndroidJavaProxy
    {
        public OnPairingsReceived onPairingsReceived;

        public GetPairingsCallback(OnPairingsReceived onPairingsReceived) : base("gg.now.wallet.sdk.listeners.GetPairingsCallback")
        {
            this.onPairingsReceived = onPairingsReceived;
        }

        public void OnPairingsReceived(string pairings)
        {
            var pairingList = JsonConvert.DeserializeObject<List<Pairing>>(pairings);
            onPairingsReceived(pairingList);
        }
    }



    class Listener : AndroidJavaProxy
    {

        public StringCallback onSuccess;
        public StringCallback onError;

        public Listener(StringCallback onSuccess, StringCallback onError) : base("gg.now.wallet.sdk.listeners.Listener")
        {
            this.onSuccess = onSuccess;
            this.onError = onError;


        }

        public void OnSuccess(string sessionTopic)
        {
            Debug.Log(sessionTopic);
            onSuccess(sessionTopic);
        }

        public void OnError(string errorMessage)
        {
            Debug.Log(errorMessage);
            onError(errorMessage);
        }
    }


}

